//-----------------------------------------------
// Author: Michael Laenen
//-----------------------------------------------
// Since Arduino can't handle https, we need to use Pushingbox API (uses http) to run the Google Script (uses https).
#include <stdio.h>
#include <math.h>
#include <SPI.h>
#include <Ethernet2.h>
#include "DHT.h"
#include <Wire.h>
#include "Adafruit_MPL115A2.h"

#define DHTPIN 2     // what pin we're connected to
#define DHTTYPE DHT22   // DHT 22  (AM2302)

//-------------------------------------------------------------------------------
//}; //Setting MAC Address
byte mac[] = { 0xDE, 0xAD, 0xBE, 0xEF, 0xFE, 0xED 
};
byte my_ip[] = { 213, 189, 176, 208 
}; //Arduino IP address. Only used when DHCP is turned off.

EthernetClient client; //define 'client' as object
char server[] = "api.pushingbox.com"; //pushingbox API server

String data; //GET query with data
boolean isConnected = false;

int led = 7; //led is pin on digital-7
int maxHum = 60;
int maxTemp = 40;
 
DHT dht(DHTPIN, DHTTYPE);
Adafruit_MPL115A2 mpl115a2 = Adafruit_MPL115A2();

// need to be declared on full scope
float h;
float t;
float p;
char hum[10];
char temp[10];

//------------------------------------------------------------------------------
void setup() {
  pinMode(led, OUTPUT); 
  Serial.begin(9600);
  Serial.println("... Initializing ethernet");
  if (Ethernet.begin(mac) == 0) {
	Serial.println("Failed to configure Ethernet using DHCP");
	Ethernet.begin(mac, my_ip);
  }
  Serial.println("... Done initializing ethernet");
  Serial.println();
  delay(1000);
  Serial.println("Getting DHT measures ...");
  dht.begin();
  Serial.println("Getting barometric pressure ...");
  mpl115a2.begin();
  
}
//------------------------------------------------------------------------------
void loop(){
  
   h = dht.readHumidity();
   t = dht.readTemperature();
   p = mpl115a2.getPressure()*10; // in kPa
   
   Serial.print("Pressure (kPa): "); Serial.print(p); Serial.println(" hPa");
   
   // Check if any reads failed and exit early (to try again).
   if (isnan(h) || isnan(t)) {
    Serial.println("Failed to read from DHT sensor!");
    return;
   }
  
   if(h > maxHum || t > maxTemp) {
      digitalWrite(led, HIGH);
   } else {
     digitalWrite(led, LOW); 
   }
  
   createQueryString(); //packing GET query with data
   Serial.println(data);
   
   Serial.println("Connecting ...");
   if (client.connect(server, 80)) {
     sendData();  
     isConnected = true; 
   }
   else{
     Serial.println("Connection failed");
   }
  // loop
  while(isConnected){
    if (client.available()) {
	  char c = client.read(); //save http header to c
	  Serial.print(c); //print http header to serial monitor
    }
    if (!client.connected()) {
	  Serial.println();
	  Serial.println("Disconnecting.");
          Serial.print("Humidity: "); 
          Serial.print(h);
          Serial.print(" %\t");
          Serial.print("Temperature: "); 
          Serial.print(t);
          Serial.println(" *C ");
          
	  client.stop(); 
          isConnected = false; 
          data = ""; //reset
    }
  }
  delay(300000); // interval 5min
}

void createQueryString(){
  double tempint, tempfrac;
  double humint, humfrac;
  double pint, pfrac;
  tempfrac =100*modf(t, &tempint);
  humfrac = 100*modf(h, &humint);
  pfrac = 1000*modf(p, &pint);

  data+="";
  data+="GET /pushingbox?devid=v28A0B0D25D11574&tempData="; //GET request query to pushingbox API
  data+=(int)tempint;
  data+=".";
  data+=(int)tempfrac;
  data+="&humData=";
  data+=(int)humint;
  data+=".";
  data+=(int)humfrac;
  data+="&pData=";
  data+=(int)pint;
  data+=".";
  data+=(int)pfrac;
  data+=" HTTP/1.1";
}
void sendData(){
  Serial.println("connected");
  client.println(data);
  client.println("Host: api.pushingbox.com");
  client.println("Connection: close");
  client.println();
}
